package com.me.pacman.state;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Input;
import com.badlogic.gdx.InputAdapter;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.graphics.g2d.BitmapFont;
import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.me.pacman.PacDude;
import com.me.pacman.entity.Direction;
import com.me.pacman.entity.Ghost;
import com.me.pacman.entity.Pacman;
import com.me.pacman.entity.ai.*;
import com.me.pacman.level.Level;
import com.me.pacman.level.LevelTile;

import java.util.Random;

public class PlayState extends LevelState {

    public static final Target BLINKY_SCATTER_TARGET = new Target((PacDude.LEVEL_WIDTH / 8) - 2, PacDude.LEVEL_HEIGHT / 8);
    public static final Target PINKY_SCATTER_TARGET = new Target(2, PacDude.LEVEL_HEIGHT / 8);
    public static final Target INKY_SCATTER_TARGET = new Target(PacDude.LEVEL_WIDTH / 8, -1);
    public static final Target CLYDE_SCATTER_TARGET = new Target(1, -1);

    private Texture levelBackground;
    private BitmapFont font;

    private TextureRegion lifeSprite;

    public Random random;

    private long sirenId;

    private int pelletCount;
    private int pelletEatenCount;
    private int score;
    private int lives;
    private int round;
    private boolean paused = false;

    private float readyTimer;
    private float gameOverTimer;
    private float newGameTimer;
    private float deathTimer;
    private float pointsTimer;
    public float frightTimer;

    private boolean pacmanCaught;
    private int ghostsCaught;
    private Ghost lastGhostCaptured;

    public Pacman pacman;
    public Ghost[] ghosts;

    public PlayState(PacDude game) {
        super(game);
    }

    @Override
    public void setup() {
        levelBackground = game.assets.getLevelBackground();
        font = game.assets.getFont();
        lifeSprite = game.assets.pacman[2][1];
        Gdx.input.setInputProcessor(new Controller());

        newGame();
    }

    @Override
    public void render() {
        game.batch.draw(levelBackground, 0, 16);
        level.render(0, 16);

        game.assets.getFont().setColor(Color.WHITE);
        game.assets.getFont().draw(game.batch, "" + score, 40, 279);
        for (int i = 0; i < lives; i++) {
            game.batch.draw(lifeSprite, i * 16, 0);
        }

        if (pointsTimer > 0) {
            game.batch.draw(game.assets.points[0][ghostsCaught-1], (lastGhostCaptured.pos.x * 8) - 8, (lastGhostCaptured.pos.y * 8) + 8);
        } else {
            pacman.render(game.batch, 0, 16);
        }

        if (pacman.alive) {
            for (Ghost ghost : ghosts) {
                if (ghost == null || (pointsTimer > 0 && ghost == lastGhostCaptured)) {
                    continue;
                }
                ghost.render(game.batch, 0, 16);
            }
        }

        if (readyTimer > 0) {
            game.assets.getFont().setColor(Color.YELLOW);
            game.assets.getFont().draw(game.batch, "ready!", 92, 127);
        }

        if (gameOverTimer > 0) {
            game.assets.getFont().setColor(Color.RED);
            game.assets.getFont().draw(game.batch, "game  over", 78, 127);
        }

        if (paused) {
            game.assets.getFont().setColor(Color.YELLOW);
            game.assets.getFont().draw(game.batch, "paused", 90, 151);
        }
    }

    @Override
    public void update(float dt) {
        if (paused) {
            return;
        }

        if (PacDude.DEBUG) {
            // Fixed time step for debugger
            dt = 1/60f;
        }

        pointsTimer -= dt;
        if (pointsTimer > 0) {
            return;
        }

        if (gameOverTimer > 0) {
            gameOverTimer -= dt;
            if (gameOverTimer <= 0) {
                newGame();
            }
            return;
        }

        if (readyTimer > 0) {
            readyTimer -= dt;
            if (readyTimer <= 0) {
                startGame();
            } else if (newGameTimer <= 0) {
                return;
            }
        }

        if (newGameTimer > 0) {
            newGameTimer -= dt;
            if (newGameTimer <= 0) {
                setupGame();
            }
            return;
        }

        if (frightTimer > 0) {
            frightTimer -= dt;
            if (frightTimer <= 0) {
                for (Ghost ghost : ghosts) {
                    if (ghost == null || ghost.currentBehaviour instanceof ReturnToBase) continue;
                    ghost.caught = false;
                    ghost.currentBehaviour = ghost.chaseBehaviour;
                    ghost.speed = Ghost.GHOST_SPEED;
                }
            }
        }

        if (!pacmanCaught) {
            for (Ghost ghost : ghosts) {
                if (ghost == null) {
                    continue;
                }
                ghost.update(dt);

                if (ghost.onSameTile(pacman)) {
                    if (frightTimer > 0 && !ghost.caught) {
                        ghostCaught(ghost);
                    } else if (frightTimer <= 0 || !(ghost.currentBehaviour instanceof ReturnToBase)) {
                        pacmanCaught();
                        return;
                    }
                }
            }
        } else if (deathTimer <= 0) {
            if (pacman.alive) {
                game.assets.deathSound.play(1f);
            }

            pacman.alive = false;

            if (pacman.deathFrame == 13) {
                if (lives > 0) {
                    setupGame();
                    readyTimer = 2f;
                } else {
                    gameOverTimer = 2f;
                }
            }
        } else {
            deathTimer -= dt;
        }

        pacman.update(dt);
    }

    public void initializeField() {
        frightTimer = 0f;
        deathTimer = 0f;
        pacmanCaught = false;
        ghostsCaught = 0;
        lastGhostCaptured = null;

        random = new Random(897198256012865L);
        ghosts = new Ghost[4];
        pacman = new Pacman(this, false);
    }

    public void newGame() {
        pelletCount = 0;
        pelletEatenCount = 0;
        score = 0;
        lives = 3;
        round = 0;

        newGameTimer = 2.3f;

        newRound();
    }

    public void newRound() {
        round++;

        level = new Level(game,"level");
        pelletCount = level.getTileCount(LevelTile.PELLET);
        pelletCount += level.getTileCount(LevelTile.POWER_PELLET);

        initializeField();
        if (newGameTimer <= 0) {
            spawnGhosts();
        }

        game.assets.siren.stop(sirenId);
        if (round == 1) {
            readyTimer = 4.2f;
            game.assets.beginning_alt.play(1.0f);
        } else {
            readyTimer = 4.3f;
            game.assets.beginning.play(1.0f);
        }
    }

    public void setupGame() {
        lives--;
        initializeField();
        spawnGhosts();
    }

    public void spawnGhosts() {
        ghosts[0] = new Ghost(this, 14, 19.5f, Direction.LEFT, 0, new BlinkyChaseBehaviour(this), new StaticTargetBehaviour(this, BLINKY_SCATTER_TARGET), false);
        ghosts[1] = new Ghost(this, 14f, 16.5f, Direction.DOWN, 1, new PinkyChaseBehaviour(this), new StaticTargetBehaviour(this, PINKY_SCATTER_TARGET), true);
        ghosts[2] = new Ghost(this, 12f, 16.5f, Direction.UP, 2, new InkyChaseBehaviour(this), new StaticTargetBehaviour(this, INKY_SCATTER_TARGET), true);
        ghosts[3] = new Ghost(this, 16f, 16.5f, Direction.UP, 3, new ClydeChaseBehaviour(this), new StaticTargetBehaviour(this, CLYDE_SCATTER_TARGET), true);

        for (int i = 0; i < 4; i++) {
            ghosts[i].currentBehaviour = ghosts[i].chaseBehaviour;
        }
    }

    public void startGame() {
        pacman.moving = true;
        sirenId = game.assets.siren.loop(1.0f);
    }

    private void pelletEaten(float x, float y) {
        level.setTile(x, y, LevelTile.EMPTY);
        if (pelletEatenCount % 2 == 0) {
            game.assets.chomp_1.play(1.0f);
        } else {
            game.assets.chomp_2.play(1.0f);
        }
        pelletEatenCount++;
        pelletCount--;

        if (pelletCount == 0) {
            newRound();
        }
    }

    public void eatPellet(float x, float y) {
        pelletEaten(x, y);
        score += 10;
    }

    public void eatPowerPellet(float x, float y) {
        pelletEaten(x, y);
        for (Ghost ghost : ghosts) {
            if (ghost == null || ghost.currentBehaviour instanceof ReturnToBase) continue;
            ghost.caught = false;
            ghost.currentBehaviour = ghost.frightBehaviour;
            ghost.currDirection = ghost.currDirection.getOpposite();
            ghost.speed = Ghost.GHOST_TUNNEL_SPEED;
        }
        frightTimer = 6f;
        ghostsCaught = 0;
        score += 50;
    }

    private void pacmanCaught() {
        game.assets.siren.stop(sirenId);
        deathTimer = 1f;
        pacmanCaught = true;
        pacman.moving = false;
    }

    private void ghostCaught(Ghost ghost) {
        ghost.caught = true;
        ghost.currentBehaviour = new ReturnToBase(this);
        ghost.speed = Ghost.EYES_SPEED;

        lastGhostCaptured = ghost;
        ghostsCaught++;
        score += ghostsCaught * 200;
        pointsTimer = 1f;
    }

    @Override
    public void dispose() {
        Gdx.input.setInputProcessor(null);
    }

    private final class Controller extends InputAdapter {

        @Override
        public boolean keyDown(int keycode) {
            switch (keycode) {
                case Input.Keys.UP:
                    pacman.setNextDirection(Direction.UP);
                    break;
                case Input.Keys.DOWN:
                    pacman.setNextDirection(Direction.DOWN);
                    break;
                case Input.Keys.LEFT:
                    pacman.setNextDirection(Direction.LEFT);
                    break;
                case Input.Keys.RIGHT:
                    pacman.setNextDirection(Direction.RIGHT);
                    break;
                case Input.Keys.P:
                    paused = !paused;
                    break;
                case Input.Keys.N:
                    newGame();
                    break;
            }
            return super.keyDown(keycode);
        }

    }

}
